package im.zego.live;

import android.app.Application;

import im.zego.zim.ZIM;

/**
 * ZIM SDK 管理类
 * <p>
 * 描述: 此类包含并管理ZIM实例
 */
public class ZegoZIMManager {

    private static volatile ZegoZIMManager singleton = null;

    private ZegoZIMManager() {
    }

    /**
     * 获取ZegoZIMManager唯一实例
     * <p>
     * 描述：这个方法用来获取ZegoZIMManager唯一实例
     * <p>
     * 调用时机：当你需要ZegoZIMManager单例的时候
     *
     * @return ZegoZIMManager 唯一实例
     */
    public static ZegoZIMManager getInstance() {
        if (singleton == null) {
            synchronized (ZegoZIMManager.class) {
                if (singleton == null) {
                    singleton = new ZegoZIMManager();
                }
            }
        }
        return singleton;
    }

    // ZIM SDK instance objects
    public ZIM zim;

    /**
     * 描述：你需要在登录、创建房间、加入房间发送消息或执行其他ZIM操作前调用此方法初始化SDK，
     * 这个方法需要结合destroyZIM方法使用，确保进程只运行一个ZIM SDK实例。
     * <p>
     * 调用时机：调用其他ZIM SDK方法之前，建议在应用启动的时候调用。
     *
     * @param appID 项目ID，请在<a href="https://console.zego.im/dashboard">控制台</a>获取。
     */
    public void createZIM(long appID, Application application) {
        zim = ZIM.create(appID, application);
    }

    /**
     * 销毁 ZIM SDK 实例
     * <p>
     * 描述：该方法用来销毁 ZIM SDK 实例和释放占用资源
     * <p>
     * 调用时机：当ZIM不再使用的时候，建议在应用退出的时候调用
     */
    public void destroyZIM() {
        if (zim != null) {
            zim.destroy();
        }
    }
}