package im.zego.live.listener;

import im.zego.live.model.OperationAction;
import im.zego.live.model.ZegoRoomInfo;
import im.zego.zim.enums.ZIMConnectionEvent;
import im.zego.zim.enums.ZIMConnectionState;

/**
 * 委托相关房间状态回调
 * <p>
 * 描述: 当房间状态变更时会触发回调。
 */
public interface ZegoRoomServiceListener {

    /**
     * 房间状态更新回调
     * <p>
     * 描述：禁用文字聊天或者房间中麦位被关闭触发该回调。所有房间里的用户通过这个回调接收通知。
     * @param roomInfo 更新房间信息。
     */
    void onReceiveRoomInfoUpdate(ZegoRoomInfo roomInfo);

    /**
     * 麦上用户更新回调
     * <p>
     * 描述：麦上用户状态更新会触发该回调，例如上麦，下麦。
     * @param action 操作行为
     */
    void onReceiveCoHostListUpdate(OperationAction action);

    /**
     * 用户连接状态回调
     * <p>
     * 描述：当用户因为网络错误或者被踢出时会触发。
     *
     * @param state 最近的连接状态。
     * @param event 造成连接状态变更的原因。
     */
    void onConnectionStateChanged(ZIMConnectionState state, ZIMConnectionEvent event);

    /**
     * Token认证信息马上就要过期时通知
     * <p>
     * 描述：当Token认证信息马上要到期时会通知，请使用renewToken更新Token认证信息。
     *
     * @param remainTimeInSecond Token马上就要过期的时间。
     * @param roomID             用户登录的房间ID
     */
    void onRoomTokenWillExpire(int remainTimeInSecond, String roomID);
}