package im.zego.live.listener;

import java.util.List;

import im.zego.live.model.ZegoUserInfo;

/**
 * 相关用户状态回调
 * <p>
 * 描述：当房间中用户状态变更会触发回调
 */
public interface ZegoUserServiceListener {

    /**
     * 用户列表更新回调
     * <p>
     * 描述：当用户状态变更该回调会被触发。
     *
     * @param userList 最近的用户列表。
     */
    void onRoomUserInfoUpdate(List<ZegoUserInfo> userList);

    /**
     * 新用户加入房间回调会被触发
     * <p>
     * 描述：当一个新用户加入房间回调会被触发，所有在房间中的用户都会收到该通知，房间的用户列表信息将会被自动更新。
     *
     * @param userList 最新加入的用户列表，不包含已经存在的用户。
     */
    void onRoomUserJoin(List<ZegoUserInfo> userList);

    /**
     * 已经存在用户退出房间回调
     * <p>
     * 描述：当房间存在用户离开房间该回调会被触发，房间所有用户都会收到该通知，房间的用户列表信息将会被自动更新。
     *
     * @param userList 离开房间的用户列表。
     */
    void onRoomUserLeave(List<ZegoUserInfo> userList);

    /**
     * 收到连麦邀请回调
     * <p>
     * 描述：当参与者在房间被房主邀请上麦会触发该回调。
     */
    void onReceiveAddCoHostInvitation(String operateUserID);

    /**
     * 连麦邀请的响应回调
     * <p>
     * 描述：当房主收到参与者回应连麦请求时会触发。
     *
     * @param accept 表示邀请参与者是否接受或拒绝邀请。
     */
    void onReceiveAddCoHostRespond(String userID, boolean accept);

    /**
     * 收到连麦请求回调
     * <p>
     * 描述：当房主收到在房间的参与者连麦请求时触发回调。
     */
    void onReceiveToCoHostRequest(String requestUserID);

    /**
     * 连麦请求取消回调
     * <p>
     * 描述：当参与者取消连麦请求，房主会触发该回调。
     */
    void onReceiveCancelToCoHostRequest(String requestUserID);

    /**
     * 连麦请求响应回调
     * <p>
     * 描述：当房主回应参与者的连麦请求时，参与者会触发该回调。
     */
    void onReceiveToCoHostRespond(boolean agree);
}