package im.zego.live.service;

import android.util.Log;
import im.zego.live.model.FaceBeautifyType;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.entity.ZegoEffectsBeautyParam;

/**
 * 美颜管理类
 * <p>
 * 描述：此类包含了应用/禁用美颜和美颜参数设置逻辑
 */
public class ZegoFaceBeautifyService {

    private static final String TAG = "Beautify";
    ZegoEffectsBeautyParam beautyParam = new ZegoEffectsBeautyParam();

    public ZegoFaceBeautifyService() {
        ZegoExpressEngine.getEngine().startEffectsEnv();
        initBeauty();
    }

    /**
     * 应用美颜功能（包括脸部美颜和脸型修整）
     * <p>
     * 描述：当调用这个方法之后，采集到的视频会在推流到其他用户前被加工
     * <p>
     * 调用时机：加入房间中
     *
     * @param enable 是否启动美颜功能。true: 启用，false：关闭。
     */
    public void enableBeautify(boolean enable) {
       ZegoExpressEngine.getEngine().enableEffectsBeauty(enable);
    }

    private void initBeauty() {
        setBeautifyValue(50, FaceBeautifyType.SkinToneEnhancement);
        setBeautifyValue(50, FaceBeautifyType.SkinSmoothing);
        setBeautifyValue(50, FaceBeautifyType.ImageSharpening);
        setBeautifyValue(5, FaceBeautifyType.CheekBlusher);
    }

    public void resetBeauty() {
        setBeautifyValue(50, FaceBeautifyType.SkinToneEnhancement);
        setBeautifyValue(50, FaceBeautifyType.SkinSmoothing);
        setBeautifyValue(50, FaceBeautifyType.ImageSharpening);
        setBeautifyValue(5, FaceBeautifyType.CheekBlusher);
    }

    public void resetReSharp() {

    }

    public int getBeautifyValue(FaceBeautifyType type) {
        int value = 0;
        switch (type) {
            case SkinToneEnhancement:
                value = beautyParam.whitenIntensity;
                break;
            case SkinSmoothing:
                value = beautyParam.smoothIntensity;
                break;
            case ImageSharpening:
                value = beautyParam.sharpenIntensity;
                break;
            case CheekBlusher:
                value = beautyParam.rosyIntensity;
                break;
            default:
                break;
        }
        Log.d(TAG, "getBeautifyValue() returned: " + value + ",type:" + type);
        return value;
    }

    /**
     * 设置美颜强度
     * <p>
     * 描述：在启用美颜之后，可以按需指定参数来设置特定美颜强度。
     * <p>
     * 调用时机：在启用美颜之后
     *
     * @param value 特定美颜强度或者脸型修整强度
     * @param type  特定的美颜类型或脸型修整类型
     */
    public void setBeautifyValue(int value, FaceBeautifyType type) {
        Log.d(TAG, "setBeautifyValue() called with: value = [" + value + "], type = [" + type + "]");
        switch (type) {
            case SkinToneEnhancement:
                beautyParam.whitenIntensity = value;
                break;
            case SkinSmoothing:
                beautyParam.smoothIntensity = value;
                break;
            case ImageSharpening:
                beautyParam.sharpenIntensity = value;
                break;
            case CheekBlusher:
                beautyParam.rosyIntensity = value;
                break;
            default:
                break;
        }
        ZegoExpressEngine.getEngine().setEffectsBeautyParam(beautyParam);
    }
}
