package im.zego.live.service;


import im.zego.live.ZegoRoomManager;
import im.zego.live.ZegoZIMManager;
import im.zego.live.callback.ZegoRoomCallback;
import im.zego.live.helper.UserInfoHelper;
import im.zego.live.listener.ZegoMessageServiceListener;
import im.zego.live.model.ZegoTextMessage;
import im.zego.live.model.ZegoUserInfo;
import im.zego.zim.ZIM;
import im.zego.zim.entity.ZIMMessage;
import im.zego.zim.entity.ZIMMessageSendConfig;
import im.zego.zim.entity.ZIMTextMessage;
import im.zego.zim.enums.ZIMErrorCode;
import im.zego.zim.enums.ZIMMessageType;
import java.util.ArrayList;
import java.util.List;

/**
 * IM消息管理类
 * <p>
 * 描述：此类包含了IM消息管理逻辑，例如发送和接受消息
 */
public class ZegoMessageService {

    /**
     * 监听消息更新
     */
    private ZegoMessageServiceListener messageServiceListener;
    /**
     * 消息列表
     */
    private List<ZegoTextMessage> messageList;

    public ZegoMessageService() {
        messageList = new ArrayList<>();
    }

    /**
     * 发送IM文字消息
     * <p>
     * 描述：这个方法可以用来发送IM文字消息，所有在房间的用户会收到这条消息提醒
     * <p>
     * 调用时机：加入房间之后
     *
     * @param text     文字消息内容，限制在1kb
     * @param callback 发送文字消息回调
     */
    public void sendTextMessage(String text, ZegoRoomCallback callback) {
        ZegoUserInfo localUserInfo = ZegoRoomManager
            .getInstance().userService.localUserInfo;
        ZegoTextMessage textMessage = new ZegoTextMessage();
        textMessage.message = text;
        textMessage.userName = UserInfoHelper.getUserName(localUserInfo.getUserID());
        textMessage.fromUserID = localUserInfo.getUserID();
        textMessage.messageTime = System.currentTimeMillis();
        String roomID = ZegoRoomManager.getInstance().roomService.roomInfo.getRoomID();
        ZegoZIMManager.getInstance().zim.sendRoomMessage(textMessage, roomID, new ZIMMessageSendConfig(),
            (message, errorInfo) -> {
                if (errorInfo.code == ZIMErrorCode.SUCCESS) {
                    messageList.add(textMessage);
                }
                if (callback != null) {
                    callback.onRoomCallback(errorInfo.code.value());
                }
            });
    }

    public void onReceiveRoomMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromRoomID) {
        for (ZIMMessage zimMessage : messageList) {
            if (zimMessage.getType() == ZIMMessageType.TEXT) {
                ZIMTextMessage zimTextMessage = (ZIMTextMessage) zimMessage;
                ZegoTextMessage textMessage = new ZegoTextMessage();
                textMessage.message = zimTextMessage.message;
                textMessage.userName = UserInfoHelper.getUserName(zimTextMessage.getSenderUserID());
                textMessage.fromUserID = zimTextMessage.getSenderUserID();
                textMessage.messageTime = zimTextMessage.getTimestamp();
                this.messageList.add(textMessage);
                if (messageServiceListener != null) {
                    messageServiceListener.onReceiveTextMessage(textMessage);
                }
            }
        }
    }

    void reset() {
        messageList.clear();
        messageServiceListener = null;
    }

    public void setListener(ZegoMessageServiceListener listener) {
        this.messageServiceListener = listener;
    }

    public List<ZegoTextMessage> getMessageList() {
        return messageList;
    }
}
